@@ -30,7 +30,8 @@
 
 #include <limits.h>
 
-static igraph_error_t igraph_i_eigenvector_centrality(igraph_real_t *to, const igraph_real_t *from,
+/* Multiplies vector 'from' by the unweighted adjacency matrix and stores the result in 'to'. */
+static igraph_error_t adjmat_mul_unweighted(igraph_real_t *to, const igraph_real_t *from,
                                            int n, void *extra) {
     igraph_adjlist_t *adjlist = extra;
     igraph_vector_int_t *neis;
@@ -46,7 +47,6 @@ static igraph_error_t igraph_i_eigenvector_centrality(igraph_real_t *to, const i
         }
     }
 
-
     return IGRAPH_SUCCESS;
 }
 
@@ -56,7 +56,8 @@ typedef struct igraph_i_eigenvector_centrality_t {
     const igraph_vector_t *weights;
 } igraph_i_eigenvector_centrality_t;
 
-static igraph_error_t igraph_i_eigenvector_centrality2(igraph_real_t *to, const igraph_real_t *from,
+/* Multiplies vector 'from' by the weighted adjacency matrix and stores the result in 'to'. */
+static igraph_error_t adjmat_mul_weighted(igraph_real_t *to, const igraph_real_t *from,
                                             int n, void *extra) {
 
     igraph_i_eigenvector_centrality_t *data = extra;
@@ -134,9 +135,10 @@ static igraph_error_t igraph_i_eigenvector_centrality_undirected(const igraph_t
         if (min < 0) {
             /* When there are negative weights, the eigenvalue and the eigenvector are no
              * longer guaranteed to be non-negative. */
-            negative_weights = 1;
+            negative_weights = true;
             IGRAPH_WARNING("Negative weight in graph. The largest eigenvalue "
-                           "will be selected, but it may not be the largest in magnitude.");
+                           "will be selected, but it may not be the largest in magnitude. "
+                           "Some eigenvector centralities may be negative.");
         }
     }
 
@@ -149,9 +151,10 @@ static igraph_error_t igraph_i_eigenvector_centrality_undirected(const igraph_t
     RNG_BEGIN();
     for (i = 0; i < no_of_nodes; i++) {
         if (VECTOR(degree)[i]) {
-            MATRIX(vectors, i, 0) = VECTOR(degree)[i] + RNG_UNIF(-1e-4, 1e-4);
+            /* Note: Keep random perturbation non-negative. */
+            MATRIX(vectors, i, 0) = VECTOR(degree)[i] + RNG_UNIF(0, 1e-4);
         } else {
-            MATRIX(vectors, i, 0) = 1.0;
+            MATRIX(vectors, i, 0) = 0.01;
         }
     }
     RNG_END();
@@ -171,7 +174,7 @@ static igraph_error_t igraph_i_eigenvector_centrality_undirected(const igraph_t
         IGRAPH_CHECK(igraph_adjlist_init(graph, &adjlist, IGRAPH_ALL, IGRAPH_LOOPS_TWICE, IGRAPH_MULTIPLE));
         IGRAPH_FINALLY(igraph_adjlist_destroy, &adjlist);
 
-        IGRAPH_CHECK(igraph_arpack_rssolve(igraph_i_eigenvector_centrality,
+        IGRAPH_CHECK(igraph_arpack_rssolve(adjmat_mul_unweighted,
                                            &adjlist, options, 0, &values, &vectors));
 
         igraph_adjlist_destroy(&adjlist);
@@ -189,7 +192,7 @@ static igraph_error_t igraph_i_eigenvector_centrality_undirected(const igraph_t
         IGRAPH_CHECK(igraph_inclist_init(graph, &inclist, IGRAPH_ALL, IGRAPH_LOOPS_TWICE));
         IGRAPH_FINALLY(igraph_inclist_destroy, &inclist);
 
-        IGRAPH_CHECK(igraph_arpack_rssolve(igraph_i_eigenvector_centrality2,
+        IGRAPH_CHECK(igraph_arpack_rssolve(adjmat_mul_weighted,
                                            &data, options, 0, &values, &vectors));
 
         igraph_inclist_destroy(&inclist);
@@ -280,8 +283,7 @@ static igraph_error_t igraph_i_eigenvector_centrality_directed(const igraph_t *g
     IGRAPH_CHECK(igraph_is_dag(graph, &dag));
     if (dag) {
         /* special case: graph is a DAG */
-        IGRAPH_WARNING("graph is directed and acyclic; eigenvector centralities "
-                       "will be zeros");
+        IGRAPH_WARNING("Graph is directed and acyclic; eigenvector centralities will be zeros.");
         if (value) {
             *value = 0;
         }
@@ -300,9 +302,6 @@ static igraph_error_t igraph_i_eigenvector_centrality_directed(const igraph_t *g
                     "number of edges (%" IGRAPH_PRId ").", IGRAPH_EINVAL,
                     igraph_vector_size(weights), igraph_ecount(graph));
         }
-        if (igraph_is_directed(graph)) {
-            IGRAPH_WARNING("Weighted directed graph in eigenvector centrality");
-        }
 
         /* Safe to call minmax, ecount == 0 case was caught earlier */
         igraph_vector_minmax(weights, &min, &max);
@@ -310,7 +309,7 @@ static igraph_error_t igraph_i_eigenvector_centrality_directed(const igraph_t *g
         if (min < 0.0) {
             /* When there are negative weights, the eigenvalue and the eigenvector are no
              * longer guaranteed to be non-negative, or even real-valued. */
-            negative_weights = 1;
+            negative_weights = true;
             IGRAPH_WARNING("Negative weights in directed graph, eigenpair may be complex.");
         }
         if (min == 0.0 && max == 0.0) {
@@ -333,7 +332,10 @@ static igraph_error_t igraph_i_eigenvector_centrality_directed(const igraph_t *g
     options->n = (int) no_of_nodes;
     options->start = 1;
     options->nev = 1;
-    options->ncv = 0;   /* 0 means "automatic" in igraph_arpack_rnsolve */
+    /* Use higher NCV than usual as long directed cycles cause convergence issues.
+     * According to numerical experiments, a cycle graph C_n tends to need about
+     * NCV = n/4 at minimum. */
+    options->ncv = no_of_nodes > 30 ? 30 : no_of_nodes;
     /* LM mode is not OK here because +1 and -1 can be eigenvalues at the
      * same time, e.g.: a -> b -> a, c -> a */
     options->which[0] = 'L' ; options->which[1] = 'R';
@@ -347,9 +349,10 @@ static igraph_error_t igraph_i_eigenvector_centrality_directed(const igraph_t *g
     RNG_BEGIN();
     for (i = 0; i < no_of_nodes; i++) {
         if (VECTOR(indegree)[i]) {
-            MATRIX(vectors, i, 0) = VECTOR(indegree)[i] + RNG_UNIF(-1e-4, 1e-4);
+            /* Note: Keep random perturbation non-negative. */
+            MATRIX(vectors, i, 0) = VECTOR(indegree)[i] + RNG_UNIF(0, 1e-4);
         } else {
-            MATRIX(vectors, i, 0) = 1.0;
+            MATRIX(vectors, i, 0) = 0.01;
         }
     }
     RNG_END();
@@ -362,8 +365,8 @@ static igraph_error_t igraph_i_eigenvector_centrality_directed(const igraph_t *g
         IGRAPH_CHECK(igraph_adjlist_init(graph, &adjlist, IGRAPH_IN, IGRAPH_LOOPS_ONCE, IGRAPH_MULTIPLE));
         IGRAPH_FINALLY(igraph_adjlist_destroy, &adjlist);
 
-        IGRAPH_CHECK(igraph_arpack_rnsolve(igraph_i_eigenvector_centrality,
-                                           &adjlist, options, 0, &values,
+        IGRAPH_CHECK(igraph_arpack_rnsolve(adjmat_mul_unweighted,
+                                           &adjlist, options, NULL, &values,
                                            &vectors));
 
         igraph_adjlist_destroy(&adjlist);
@@ -379,8 +382,8 @@ static igraph_error_t igraph_i_eigenvector_centrality_directed(const igraph_t *g
         IGRAPH_CHECK(igraph_inclist_init(graph, &inclist, IGRAPH_IN, IGRAPH_LOOPS_ONCE));
         IGRAPH_FINALLY(igraph_inclist_destroy, &inclist);
 
-        IGRAPH_CHECK(igraph_arpack_rnsolve(igraph_i_eigenvector_centrality2,
-                                           &data, options, 0, &values, &vectors));
+        IGRAPH_CHECK(igraph_arpack_rnsolve(adjmat_mul_weighted,
+                                           &data, options, NULL, &values, &vectors));
 
         igraph_inclist_destroy(&inclist);
         IGRAPH_FINALLY_CLEAN(1);
@@ -460,8 +463,8 @@ static igraph_error_t igraph_i_eigenvector_centrality_directed(const igraph_t *g
  * </para><para>
  * In the weighted case, the eigenvector centrality of a vertex is proportional
  * to the weighted sum of centralities of its neighbours, i.e.
- * <code>c_i = sum_j w_ij c_j</code>, where <code>w_ij</code> is the weight
- * of the edge connecting vertices \c i and \c j. The weights of parallel edges
+ * <code>c_j = sum_i w_ij c_i</code>, where <code>w_ij</code> is the weight
+ * of the edge connecting vertex \c i to \c j. The weights of parallel edges
  * are added up.
  *
  * </para><para>
@@ -492,6 +495,10 @@ static igraph_error_t igraph_i_eigenvector_centrality_directed(const igraph_t *g
  * parameter, see below) and checking whether the eigenvalue is very close
  * to zero.
  *
+ * </para><para>
+ * When working with directed graphs, consider using hub and authority
+ * scores instead, see \ref igraph_hub_and_authority_scores().
+ *
  * \param graph The input graph. It may be directed.
  * \param vector Pointer to an initialized vector, it will be resized
  *     as needed. The result of the computation is stored here. It can
@@ -518,6 +525,8 @@ static igraph_error_t igraph_i_eigenvector_centrality_directed(const igraph_t *g
  *
  * \sa \ref igraph_pagerank and \ref igraph_personalized_pagerank for
  *   modifications of eigenvector centrality.
+ * \ref igraph_hub_and_authority_scores() for a similar pair of measures
+ * intended for directed graphs.
  *
  * \example examples/simple/eigenvector_centrality.c
  */
