@@ -31,8 +31,7 @@ int main(void) {
 
     igraph_rng_seed(igraph_rng_default(), 9275);
 
-    igraph_erdos_renyi_game(&g1, IGRAPH_ERDOS_RENYI_GNP, 10, .3,
-                            IGRAPH_UNDIRECTED, IGRAPH_NO_LOOPS);
+    igraph_erdos_renyi_game_gnp(&g1, 10, .3, IGRAPH_UNDIRECTED, IGRAPH_NO_LOOPS);
     igraph_correlated_game(&g1, &g2, .9, .3, /* permutation=*/ 0);
 
     IGRAPH_ASSERT(igraph_vcount(&g1) == igraph_vcount(&g2));
